<form method="POST" action="">
    {{ form.hidden_tag() if form.hidden_tag }}
    {% for f in form if f.type != 'CSRFTokenField' %}
    <div>
    {{ f.label }}
    {{ f }}
    {% if f.errors %}
    <ul>
        {% for e in f.errors %}
        <li>{{ e }}</li>
        {% endfor %}
    </ul>
    {% endif %}
    </div>
    {% endfor %}
    <button class="btn" type="submit">Submit</button>
</form>
